@@ -9,9 +9,8 @@ from Bio.PDB.Entity import Entity
 
 
 class Structure(Entity):
-    """
-    The Structure class contains a collection of Model instances.
-    """
+    """The Structure class contains a collection of Model instances."""
+
     def __init__(self, id):
         self.level = "S"
         Entity.__init__(self, id)
@@ -31,7 +30,8 @@ class Structure(Entity):
         reflects the order of the models in the PDB file.
 
         Arguments:
-        o m1, m2 - Model instances
+         - m1, m2 - Model instances
+
         """
         return cmp(m1.get_id(), m2.get_id())
 
