@@ -98,7 +98,8 @@ class PrintFormat(object):
            those without sites.
          - s1 must be a formatted string as well.
 
-        The format of print_that is a list."""
+        The format of print_that is a list.
+        """
         if not dct:
             dct = self.results
         ls, nc = [], []
@@ -121,8 +122,9 @@ class PrintFormat(object):
          - s1 must be a formatted string as well.
 
         This method prints the output of A.format_output() and it is here
-        for backwards compatibility."""
-        print(format_output(dct, title, s1))
+        for backwards compatibility.
+        """
+        print(self.format_output(dct, title, s1))
         return
 
     def make_format(self, cut=(), title='', nc=(), s1=''):
@@ -151,6 +153,7 @@ class PrintFormat(object):
          - title is the title.
          - nc is a tuple or list of non cutting enzymes.
          - s1 is the sentence before the non cutting enzymes.
+
         """
         return self._make_list_only(ls, title) + self._make_nocut_only(nc, s1)
 
@@ -172,6 +175,7 @@ class PrintFormat(object):
          - title is the title.
          - nc is a list of non cutting enzymes.
          - s1 is the sentence before the non cutting enzymes.
+
         """
         return self._make_map_only(ls, title) + self._make_nocut_only(nc, s1)
 
@@ -196,6 +200,7 @@ class PrintFormat(object):
          - title is the title.
          - nc is a list of non cutting enzymes.
          - s1 is the sentence before the non cutting enzymes.
+
         """
         return self._make_number_only(ls, title) + self._make_nocut_only(nc, s1)
 
@@ -211,6 +216,7 @@ class PrintFormat(object):
          - title is the title.
          - nc is a list of non cutting enzymes.
          - s1 is the sentence before the non cutting enzymes.
+
         """
         return title + self._make_nocut_only(nc, s1)
 
@@ -222,6 +228,7 @@ class PrintFormat(object):
         Arguments:
          - nc is a tuple or list of non cutting enzymes.
          - s1 is the sentence before the non cutting enzymes.
+
         """
         if not nc:
             return s1
@@ -251,6 +258,7 @@ class PrintFormat(object):
          - ls is a tuple or list of results.
          - title is a string.
          - Non cutting enzymes are not included.
+
         """
         if not ls:
             return title
@@ -276,11 +284,11 @@ class PrintFormat(object):
          - ls is a list of results.
          - title is a string.
          - Non cutting enzymes are not included.
+
         """
         if not ls:
             return title
-        # TODO: Use key to sort!
-        ls.sort(lambda x, y: cmp(len(x[1]), len(y[1])))
+        ls.sort(key=lambda x: len(x[1]))
         iterator = iter(ls)
         cur_len = 1
         new_sect = []
@@ -312,6 +320,7 @@ class PrintFormat(object):
          - ls is a list of results.
          - title is a string.
          - Non cutting enzymes are not included.
+
         """
         if not ls:
             return title
@@ -406,7 +415,7 @@ class PrintFormat(object):
 # private method to do lists:
 
     def __next_section(self, ls, into):
-        """FP.__next_section(ls, into) -> string.
+        """FP.__next_section(ls, into) -> string (PRIVATE).
 
         Arguments:
          - ls is a tuple/list of tuple (string, [int, int]).
